var progressFlow = require('./progressFlow');

function makeFinishActivity (username, startTime) {
    return {
        activityId: 'finish',
        activityType: 'finish',
        state: 'pristine',
        stateLastChanged: startTime,
        gotos: []
    };
}

function makeStartActivity (activities, username, startTime) {
    var j, k,
        activity,
        candidate,
        goto,
        hasAGoto,
        startActivity = {
            activityId: 'start',
            activityType: 'start',
            state: 'pristine',
            stateLastChanged: startTime,
            gotos: [],
            audience: [username]
        };

    for (var i=0; i<activities.length; i++) {
        activity = activities[i];
        hasAGoto = false;

        for (j=0; j<activities.length;j++) {
            candidate = activities[j];
            for (k=0; k<candidate.gotos.length;k++) {
                goto = candidate.gotos[k];
                if (goto.targetActivityId == activity.activityId ) {
                    hasAGoto = true;
                }
            }
        }


        if (!hasAGoto) {

            // No other activity goes to this activity,
            // so it's implied it runs from the start...
            startActivity.gotos.push(
                {
                    state: 'finished',
                    targetActivityId: activity.activityId
                }
            );
        }
    }
    return startActivity;
}

var generateFlowDoc;
module.exports.generateFlowDoc = generateFlowDoc = function generateFlowDoc(
    TemplateModel,
    FlowsModel,
    username,
    templateId,
    flowParams,
    callback) {

    TemplateModel.findOne(
        {templateId: templateId},
        function(err, templateDoc) {

            var activity;

            if (err) {
                callback(
                    {
                        status: 500,
                        error: err
                    }
                );
            }
            else {

                if (templateDoc) {


                    var templateActivities = JSON.parse(JSON.stringify(templateDoc.templateActivities)),
                        startTime = Date.now(),
                        hasStart = false;

                    for (var i=0;i<templateActivities.length;i++) {
                        activity = templateActivities[i];
                        if (activity.activityType == 'start') {
                            hasStart = true;
                        }
                        activity.state = 'pristine';
                    }

                    if (!hasStart) {
                        templateActivities.push(makeStartActivity(templateActivities, username, startTime));
                    }

                    templateActivities.push(makeFinishActivity(username, startTime));

                    var fields = {
                        state: 'pristine',
                        templateId: templateId,
                        namespace: templateDoc.namespace,
                        blueprintName: templateDoc.blueprintName,
                        blueprintVersion: templateDoc.blueprintVersion,
                        blueprintLocalVersion: templateDoc.blueprintLocalVersion,
                        context: {
                            public: {},
                            private: {}
                        },
                        activities: templateActivities,
                        createdBy: username
                    };
                    if (flowParams) {
                        fields.params = flowParams;
                    }

                    var flowDoc = new FlowsModel(fields);

                    FlowsModel.create(flowDoc, function(err, savedDoc) {
                        if (err) {
                            callback(
                                {
                                    status: 500,
                                    error: err
                                }
                            );
                        }
                        else {
                            callback(null, savedDoc);
                        }
                    });
                }
                else {
                    callback(
                        {
                            status: 404,
                            error: {
                                name: 'noTemplateDoc',
                                message: "Unable to find flow template with id '" + templateId + "'"
                            }
                        }
                    )
                }
            }

        }
    );


};



module.exports.createFlowFromRequest = function createFlowFromRequest(roadie) {

    var TemplateModel = null,
        FlowsModel = null;

    return function createFlow(req, res) {

        if (!TemplateModel) {
            TemplateModel = roadie.db.model('_sys.flow.1.0.templates');
            FlowsModel = roadie.db.model('_sys.flow.1.0.flows');
        }




        roadie.cache.get('userPasses', req.user.userId, null, function (err, userPasses) {

            if (err) {
                res.status(500).send(err);
            }
            else {

                if (userPasses.canStartFlow(req.body.templateId)) {

                    generateFlowDoc(
                        TemplateModel,
                        FlowsModel,
                        req.user.userId,
                        req.body.templateId,
                        req.body.params,
                        function(err, flowDoc) {

                            if (err) {
                                res.status(err.status).send(err.error);
                            }
                            else {
                                progressFlow(roadie, true, flowDoc, req.body.templateId, req.body.params, req.body.userId, res);
                            }


                        }
                    );
                }
                else {
                    res.status(404).send(
                        {
                            name:'noGoFlow',
                            message: 'Unable to start flow from template ' + req.body.templateId + ' - possibly no pass provides the necessary permission?'
                        }
                    );
                }

            }
        });
    }
};