var us =require('underscore'),
    common = require('./common'),
    findActivity = common.findActivity,
    sendError = common.sendError,
    setReady = common.setReady,
    progressFlow = require('./progressFlow');

module.exports = function(roadie) {

    var FlowsModel = null;

    return function setDataAndProgress(req, res) {

        var targetActivity,
            flowId = req.params.flowId,
            now = Date.now();

        if (!FlowsModel) {
            FlowsModel = roadie.db.model('_sys.flow.1.0.flows');
        }

        findActivity(
            FlowsModel,
            flowId,
            req.params.activityId,
            res,
            function(err, flowDoc, activity) {

                if (err) {
                    sendError(
                        flowId,
                        err.name,
                        err.message,
                        err.body,
                        err.status,
                        res
                    );
                }
                else {

                    if (activity.state == 'pending') {

                        targetActivity = flowDoc.findActivity(req.params.activityId);

                        if (targetActivity) {

                            if (us.has(targetActivity,'data')) {
                                targetActivity.data = {};
                            }
                            targetActivity.data = req.body;

                            flowDoc.markModified('activities');

                            flowDoc.modified = now;
                            activity.state = 'finished';
                            activity.lastCompleted = now;

                            setReady('finished', activity, flowDoc.activities);

                            flowDoc.save(function(err, savedDoc) {
                                if (err) {
                                    sendError(
                                        flowId,
                                        'badDataUpdate',
                                        "Unable to update data for activity '" + activity.activityId + "'",
                                        err,
                                        500,
                                        res
                                    );
                                }
                                else {

                                    progressFlow(
                                        roadie,
                                        true,
                                        savedDoc,
                                        savedDoc.templateId,
                                        req.params,
                                        req.user.username,
                                        res);

                                }
                            });

                        }

                        else {
                            sendError(
                                flowId,
                                'badActivity',
                                "Unable to update data for activity '" + activity.activityId + "'",
                                err,
                                404,
                                res
                            );
                        }


                    }
                    else {

                        sendError(
                            flowId,
                            'badState',
                            "Unable to update data for activity '" + activity.activityId + "' as it has a state of '"+ activity.state +"'",
                            err,
                            500,
                            res
                        );
                    }


                }

        });


    }

};