var fs = require('fs'),
    path = require('path'),
    us = require('underscore'),
    util = require('util');


var BaseActivity = function BaseActivity(path, activityStructure) {
    this._path = path;
    this.renderer = {};
    for (var id in activityStructure) {
        if (activityStructure.hasOwnProperty(id) ) {
            this[id] = activityStructure[id];
        }
    }
};


module.exports.addActivity = function addActivity (name, fromPath, relativePath) {

    var rawActivity = {
            properties: {}
        },
        activityDir = path.resolve(fromPath, relativePath),
        validationSchemaPath,
        runFunctionPath,
        responseFunctionPath,
        propertiesPath,
        properties;


    if ( fs.existsSync(activityDir) && fs.lstatSync(activityDir).isDirectory() ) {

        // Add validation schema (if there is one)
        // ---------------------------------------
        validationSchemaPath = path.join(activityDir, 'validator.json');
        if (fs.existsSync(validationSchemaPath)) {
            rawActivity.validationSchema = require(validationSchemaPath);
        }


        // Add properties
        // --------------
        propertiesPath = path.join(activityDir, 'properties.json');
        if (fs.existsSync(propertiesPath)) {
            properties = require(propertiesPath);
            for (var key in properties) {
                if (properties.hasOwnProperty(key)) {
                    rawActivity.properties[key] = properties[key];
                }
            }
        }


        // Add run function (if there is one)
        // -------------------------------------
        runFunctionPath = path.join(activityDir, 'run.js');
        if (fs.existsSync(runFunctionPath)) {
            rawActivity.runFunction = require(runFunctionPath);
        }

        // Add response function (if there is one)
        // ---------------------------------------
        responseFunctionPath = path.join(activityDir, 'response.js');
        if (fs.existsSync(responseFunctionPath)) {
            rawActivity.responseFunction = require(responseFunctionPath);
        }

        this.activities[name] = rawActivity;

        this.logger.debug("     Added '%s' activity", name);
    }
    else {
        this.logger.error("     Can't add activity from path %s", activityDir);
    }
};




module.exports.BaseActivity = BaseActivity;


