var us = require('underscore');


// states = ['pristine', 'running', 'pending', 'ready', 'failed', 'finished'];


function post (activity, context, params, body, callback) {

    activity.messages = [];

    context.roadie.data.runners.model(
        context.roadie,
        'post',
        params, //params
        {}, // query
        body, // body
        context.roadie.data.actionFunctions.post,

        function(err) {
            if (err) {
                activity.state = 'failed';
                activity.messages.push(
                    {
                        level: 'error',
                        name: err.name,
                        message: err.message,
                        body: err.errors
                    }
                );

                callback(null, activity);
            }
            else {
                activity.state = 'finished';
                callback(null, activity);
            }
        }
    );
}



function put (activity, context, params, query, rawBody, callback) {

    activity.messages = [];

    var body = JSON.parse(JSON.stringify(rawBody));


    params.docId = body[activity.config.docIdPath];
    delete body[activity.config.docIdPath];

    context.roadie.data.runners.doc(
        context.roadie,
        'put',
        params,
        query,
        body,
        context.roadie.data.initFunctions.put,
        context.roadie.data.actionFunctions.put,
        function(err) {
                if (err) {
                    activity.state = 'failed';
                    activity.messages.push(
                        {
                            level: 'error',
                            name: err.name,
                            message: err.message,
                            body: err.body
                        }
                    );

                    callback(null, activity);
                }
                else {
                    activity.state = 'finished';
                    callback(null, activity);
                }
        }
    );
}


function patch (activity, context, params, query, rawBody, callback) {

    activity.messages = [];

    var body = JSON.parse(JSON.stringify(rawBody));


    params.docId = body[activity.config.docIdPath];
    delete body[activity.config.docIdPath];

    context.roadie.data.runners.patch(
        context.roadie,
        params,
        query,
        body,
        false,
        function(err) {
            if (err) {
                activity.state = 'failed';
                activity.messages.push(
                    {
                        level: 'error',
                        name: err.name,
                        message: err.message,
                        body: err.errors
                    }
                );

                callback(null, activity);
            }
            else {
                activity.state = 'finished';
                callback(null, activity);
            }
        }
    );
}


module.exports = function runStart(activity, context, callback) {

    var targetId = context.activity.config.sourceDataActivityId,
        targetActivity = context.flowDoc.findActivity(targetId),
        body,
        method = activity.config.method,
        params =  {
            ns: context.flowDoc.namespace,
            bp: context.flowDoc.blueprintName,
            model: context.activity.config.schema
        };

    if (targetActivity) {

        body = targetActivity.data;

        switch (method) {
            case 'post':
                post (activity, context, params, body, callback);
                break;
            case 'put':
                put (activity, context, params, {}, body, callback);
                break;
            case 'patch':
                patch(activity, context, params, {}, body, callback);
                break;
        }

    }

    else {

        activity.state = 'failed';

        activity.messages.push(
            {
                level: 'error',
                name: 'noActivity',
                message: "Unknown activity '" + targetId + "'"
            }
        );

        callback(null, activity)
    }
};
