module.exports.uploadFunction = function (
    roadie,
    rawElement,
    namespace,
    blueprintName,
    blueprintVersion,
    localVersion,
    rawElements,
    elementTypes,
    logger,
    errors) {

    return function categoryUpload(restructured, callback) {

        var Categories = roadie.db.model('_sys.flow.1.0.categories'),
            self = this;


        Categories.findOne({categoryId: rawElement.id}, function(err, doc) {

            if (err) {
                errors.push(
                    {
                        name:'categoryUploadFail1',
                        message: "Problem finding category via ID",
                        body: JSON.stringify(err)
                    }
                );
                callback(null, restructured);
            }
            else {

                if (doc) {

                    if (
                        doc.namespace == namespace &&
                        doc.blueprintName == blueprintName &&
                        doc.blueprintVersion == blueprintVersion &&
                        doc.blueprintLocalVersion == localVersion
                    ) {
                        // Category already exists (and was created by this blueprint) so just move on...
                        callback(null, restructured);
                    }

                    else {
                        // Category already exists, but has already been created via another blueprint
                        // So... if it's not already, register it as an "alternative".
                        // The idea being if more that one Blueprint wants to install the same-named category
                        // at least that's registered somewhere. And an administrator could
                        // pick the "best" via a control panel or something, in case of conflict.
                        var alternative, addAlternative = true;
                        for (var i=0;i<doc.alternatives.length;i++) {
                            alternative = doc.alternatives[i];
                            if (
                                alternative.namespace == namespace &&
                                alternative.blueprintName == blueprintName &&
                                alternative.blueprintVersion == blueprintVersion &&
                                alternative.blueprintLocalVersion == localVersion
                            ) {
                                addAlternative = false;
                                break;
                            }

                        }

                        if (addAlternative) {
                            doc.alternatives.push({
                                label: rawElement.config.label,
                                description: rawElement.config.description,
                                less: rawElement.config.less,
                                namespace: namespace,
                                blueprintName: blueprintName,
                                blueprintVersion: blueprintVersion,
                                blueprintLocalVersion: localVersion
                            });
                            doc.save(
                                function(err) {
                                    if (err) {
                                        errors.push(
                                            {
                                                name:'categoryUploadFail2',
                                                message: "Problem updating category doc",
                                                body: JSON.stringify(err)
                                            }
                                        );
                                        callback(null, restructured);

                                    }
                                    else {
                                        callback(null, restructured);
                                    }
                                }
                            );

                        }
                        else {
                            callback(null, restructured);
                        }
                    }

                }
                else {

                    // Counter not installed, add doc
                    var category = new Categories ({
                        categoryId: rawElement.id,
                        label: rawElement.config.label,
                        description: rawElement.config.description,
                        less: rawElement.config.less,
                        namespace: namespace,
                        blueprintName: blueprintName,
                        blueprintVersion: blueprintVersion,
                        blueprintLocalVersion: localVersion
                    });

                    category.save(
                        function(err) {
                            if (err) {
                                errors.push(
                                    {
                                        name:'categoryUploadFail1',
                                        message: "Problem finding category via ID",
                                        body: JSON.stringify(err)
                                    }
                                );
                                callback(null, restructured);

                            }
                            else {
                                callback(null, restructured);
                            }
                        }
                    );
                }
            }
        });

    }
};
