var us = require('underscore');

function makeGotos(activityElement) {

    var key, goto, candidateElement, gotos = [];
    for (var i=0; i<activityElement._orderedElementKeys.length;i++) {
        key = activityElement._orderedElementKeys[i];
        candidateElement = activityElement._elements[key];

        if (candidateElement._elementType == 'goto') {

            goto = {
                state: candidateElement.state,
                targetActivityId: candidateElement.targetActivityId
            };

            if (us.has(candidateElement, 'condition')) {
                goto.condition = candidateElement.condition
            }

            gotos.push(goto);
        }


    }
    return gotos;
}

var makeTemplateActivities;
module.exports.makeTemplateActivities = makeTemplateActivities = function makeTemplateActivities(flowElements) {

    var key, candidate, activity, activities = [];
    for (var i=0; i<flowElements._orderedElementKeys.length;i++) {
        key = flowElements._orderedElementKeys[i];
        candidate = flowElements._elements[key];

        if (candidate._elementType=='activity') {

            activity = {
                activityId: candidate._id,
                activityType: candidate.activityType,
                config: candidate.config,
                gotos: makeGotos(candidate)
            };
            activities.push(activity);

        }
    }

    return activities;

};


module.exports.uploadFunction = function (
    roadie,
    rawElement,
    namespace,
    blueprintName,
    blueprintVersion,
    localVersion,
    rawElements,
    elementTypes,
    logger,
    errors) {



    return function flowUpload(restructured, callback) {

        if ( namespace=='_sys' && blueprintName == 'flow') {

            // We'll add the flow template for making flows later via a System-blueprint
            // so moving on...
            callback(null, restructured);
        }

        else {

            var flowId = [
                namespace,
                blueprintName,
                rawElement.id
            ].join('.');

            var Templates = roadie.db.model('_sys.flow.1.0.templates');


            function saveDoc() {

                var modelRunner = roadie.data.runners.model;
                var templateActivities = makeTemplateActivities(restructured._elements[rawElement.id]);

                modelRunner(
                    roadie,
                    'post',
                    {
                        ns: '_sys',
                        bp: 'flow',
                        model: 'templates'
                    }, // params
                    {}, // query
                    {
                        templateId: flowId,
                        namespace: namespace,
                        blueprintName: blueprintName,
                        blueprintVersion: blueprintVersion,
                        blueprintLocalVersion: localVersion,
                        name: rawElement.id,
                        category: rawElement.config.category,
                        label: rawElement.config.label,
                        description: rawElement.config.description,
                        templateActivities: templateActivities,
                        instigators: rawElement.config.instigators
                    }, // body
                    roadie.data.actionFunctions.post,

                    function(err) {
                        if (err) {
                            errors.push(
                                {
                                    name: 'TemplatesaveFail',
                                    message: "Problem saving flowTemplate",
                                    body: JSON.stringify(err)
                                }
                            );
                            logger.error("Can't save flowTemplate", JSON.stringify(templateActivities, null, 2));
                            callback(null, restructured);

                        }
                        else {
                            callback(null, restructured);
                        }
                    }
                );


            }

            Templates.findOne({templateId: flowId}, function(err, doc) {
                if (err) {
                    errors.push(
                        {
                            name:'flowTemplateFindFail',
                            message: "Problem finding flowTemplate ID",
                            body: JSON.stringify(err)
                        }
                    );
                    callback(null, restructured);

                }
                else {

                    if (doc) {

                        // A flowTemplate already exists, if this is more up-to-date,
                        // then replace it (i.e. all new flows will now be built from
                        // this instead...

                        if (
                            (blueprintVersion > doc.blueprintVersion) ||
                            ((blueprintVersion == doc.blueprintVersion) && localVersion >= doc.blueprintLocalVersion)
                        ) {

                            Templates.findOneAndRemove({templateId: flowId}, function(err) {
                                if (err) {
                                    errors.push(
                                        {
                                            name:'flowTemplateRemoveFail',
                                            message: "Problem removing flowTemplate via ID",
                                            body: JSON.stringify(err)
                                        }
                                    );
                                    callback(null, restructured);

                                }
                                else {
                                    saveDoc();
                                }
                            });

                        }
                        else {

                            // This isn't anything new, so just carry on...
                            callback(null, restructured);
                        }

                    }

                    else {

                        // No flowTemplate, so make this the new template doc...
                        saveDoc();
                    }
                }
            });


        }



    }
};
