var generateRoutes = require('./generateRoutes'),
    activities = require('./activities'),
    categoriesFlow = require('./ensure/maintainCategoriesFlow'),
    templatesFlow = require('./ensure/maintainTemplatesFlow'),
    systemCategory = require('./ensure/systemCategory'),
    grabLists = require('./activities/grabLists/grabListsCacheFunction');

module.exports.pluginName = 'roadiejs-flow';
module.exports.startupSeq = 22;

module.exports.getStartupFunction = function(roadie, config) {
    return function flowStartup (callback) {
        roadie.logger.debug('  [roadie-flow]');

        roadie.activities = {};
        roadie.BaseActivity = activities.BaseActivity;
        roadie.addActivity = activities.addActivity;

        roadie.addElementType('category', __dirname, './elements/category');
        roadie.addElementType('flow', __dirname, './elements/flow');
        roadie.addElementType('goto', __dirname, './elements/goto');
        roadie.addElementType('activity', __dirname, './elements/activity');

        roadie.addActivity('start', __dirname, './activities/start');
        roadie.addActivity('finish', __dirname, './activities/finish');
        roadie.addActivity('saveData', __dirname, './activities/saveData');
        roadie.addActivity('grabBlueprintData', __dirname, './activities/grabBlueprintData');
        roadie.addActivity('setDefaults', __dirname, './activities/setDefaults');
        roadie.addActivity('grabLists', __dirname, './activities/grabLists');

        roadie.cache.add('listSets', 500, null, grabLists);


        roadie.addSysMethod(
            'findActivity',

            function (roadie, augmentSchema, mongooseSchema) {

                mongooseSchema.methods.findActivity = function findActivity(activityId) {
                    for (var i = 0; i < this.activities.length; i++) {
                        if (this.activities[i].activityId == activityId) {
                            return this.activities[i];
                        }
                    }
                }

            }

        );


        generateRoutes(roadie);

        roadie.registerSysBlueprint(
            20,
            module.exports.pluginName,
            __dirname,
            './blueprint',
            function (roadie, callback) {

                roadie.TemplateModel = roadie.db.model('_sys.flow.1.0.templates');
                roadie.FlowsModel = roadie.db.model('_sys.flow.1.0.flows');
                roadie.FlowArchive = roadie.db.model('_sys.flow.1.0.flowArchive');
                roadie.CategoriesModel = roadie.db.model('_sys.flow.1.0.categories');

                roadie.addMiniCrudRoutes(
                    'domain', // scope
                    'categories', // domain
                    '_sys', // ns
                    'flow', // bp
                    'categories', // model
                    ['post', 'list', 'purge', 'get', 'delete'], // actions
                    'categoryId', // keyField
                    null, // defaultFunction,
                    true, // lowercaseKey,
                    null); // postUpdateFunction

                roadie.logger.debug('      categories:');
                roadie.publishRoutes('categories');


                systemCategory(roadie, function(err) {
                    templatesFlow(roadie, function(err) {
                        categoriesFlow(roadie, callback);
                    });
                });

            }
        );

        callback(null);
    };
};


module.exports.shutdownSeq=15;
module.exports.getShutdownFunction = function(roadie, config) {
    return function flowShutdown (callback) {
        roadie.logger.debug('  [roadie-flow]');
        callback(null);
    }
};