var expect =require('expect.js'),
    Roadie = require('roadiejs'),
    getOptions = require('roadiejs-util').getRouteOptions,
    flowPlugin = require('./../lib/roadiejs-flow'),
    us = require('underscore'),
    config = require('./config/default.json'),
    app = new Roadie(config),
    baseUrl = 'http://localhost:3006/',
    usersUrl = baseUrl +'users',
    categoryUrl = baseUrl + 'categories/',
    suffix = '?retainHistory=false',
    loginUrl = baseUrl + 'login',
    rest = require('restler'),
    username = 'flowTester',
    password = '123testing',
    token;


describe ('Setup tests (flow)', function() {

    this.timeout(10000);

    it ("should add the 'flow' plugin", function(done) {
        app.addPlugin(
            flowPlugin.pluginName,
            flowPlugin.startupSeq,
            flowPlugin.getStartupFunction(app, config.plugins['roadiejs-flow']),
            flowPlugin.shutdownSeq,
            flowPlugin.getShutdownFunction(app, config.plugins['roadiejs-flow'])
        );
        app.start(done);
    });


    it("should ensure user 'flowTester'", function (done) {
        app.ensureUser( {
            userId: username,
            password: password,
            passes: [
                'tester',
                'roadietest.advancedhruser'
            ]
        }, true, function(err) {
            expect(err).to.be(null);
            done();
        });
    });


    it ("should force the school blueprint to upload", function(done){
        app.uploadBlueprint(
            './blueprint',
            {
                force: true,
                recurse: true,
                fromPath: __dirname
            },
            function(err, result) {
                expect(result).not.to.be(null);
                if (result) {
                    expect(result.blueprintId).to.be('roadietest.flow.1.0');
                    expect(result._created).to.be.ok();
                    expect(result._modified).to.be.ok();
                    expect(result.namespace).to.be('roadietest');
                    expect(result.name).to.be('flow');
                    expect(result.blueprintVersion).to.be(1);
                    expect(result.localVersion).to.be(0);
                    expect(result.label).to.be('Roadie Flow');
                }
                done();
            }
        );
    });


    it ("should get ourselves a token", function(done){
        rest.postJson(loginUrl, {
            userId: username,
            password: password
        }).on('complete', function(data, res) {
            token = data.token;
            expect(res.statusCode).to.be(201);
            done();
        });
    });

});

describe ('Basic category tests', function() {


    it ("should get the 'employees' category", function(done){
        rest.get(categoryUrl + 'employees', getOptions(token)).on('complete', function(data, res) {
            expect(res.statusCode).to.be(200);
            expect(data.categoryId).to.be('employees');
            done();
        });
    });

    it ("should list categories", function(done){
        rest.get(categoryUrl, getOptions(token)).on('complete', function(data, res) {
            expect(res.statusCode).to.be(200);
            expect(data.length).to.be.greaterThan(0);
            done();
        });
    });

    it ("should delete 'testStuff' category", function(done){
        rest.del(categoryUrl + 'testStuff'+suffix, getOptions(token)).on('complete', function(data, res) {
            expect(res.statusCode).not.to.be(500);
            done();
        });
    });

    it ("should post 'testStuff' category", function(done){
        rest.postJson(categoryUrl, {
            categoryId : 'testStuff',
            label: 'Test category',
            description: "It won't last long",
            namespace: "roadietest",
            less: {
                "base-color": '#FF0000'
            },
            blueprintName: "pretendCategory",
            blueprintVersion: 1,
            blueprintLocalVersion: 0
        }, getOptions(token)).on('complete', function(data, res) {
            expect(res.statusCode).to.be(201);
            done();
        });
    });


    it ("should get the 'testStuff' category", function(done){
        rest.get(categoryUrl + 'testStuff', getOptions(token)).on('complete', function(data, res) {
            expect(res.statusCode).to.be(200);
            delete data._id;
            delete data.__v;
            delete data._bpv;
            delete data._created;
            delete data._deleted;
            delete data._latest;
            delete data._lv;
            delete data._modified;
            delete data._version;

            expect(data).to.eql({
                categoryId : 'teststuff',
                label: 'Test category',
                description: "It won't last long",
                namespace: "roadietest",
                less: {
                    "base-color": '#FF0000'
                },
                blueprintName: "pretendCategory",
                blueprintVersion: 1,
                blueprintLocalVersion: 0
            });
            done();
        });
    });

    it ("should list at least two categories", function(done){
        rest.get(categoryUrl, getOptions(token)).on('complete', function(data, res) {
            expect(res.statusCode).to.be(200);
            expect(data.length).to.be.greaterThan(1);
            done();
        });
    });

    it ("should delete 'testStuff' category", function(done){
        rest.del(categoryUrl + 'testStuff'+suffix, getOptions(token)).on('complete', function(data, res) {
            expect(res.statusCode).to.be(200);
            done();
        });
    });

    it ("shouldn't get the 'testStuff' category", function(done){
        rest.get(categoryUrl + 'testStuff', getOptions(token)).on('complete', function(data, res) {
            expect(res.statusCode).to.be(404);
            expect(data).to.eql(
                {
                    status: 404,
                    name: 'noDoc',
                    message: 'Unable to find document with conditions: {"_latest":true,"_deleted":false,"categoryId":"teststuff"}'
                }
            );
            done();
        });
    });

});

describe ('Basic flow tests', function () {

    it("should get a user's passes (just names, attempt 1)", function (done) {
        rest.get(usersUrl+'/flowtester/passes?content=full', getOptions(token)).on('complete', function (data, res) {
            expect(res.statusCode).to.be(200);
            done();
        });
    });

});

describe("Shutdown", function() {
    it ("Shutdown app", function(done) {
        app.stop(function (err) {
            expect(err).to.be(null);
            done();
        });
    });
});

//describe ('Pause before ending...', function(done) {
//    it ("Pause", function(done) {
//        setTimeout(done, 3000);
//    });
//});